@@ -1,10 +1,10 @@
 #!/bin/bash
-set -eu
+set -xeu
 
 declare -A PKG_MAP
 
 # workaround: for latest bindep to work, it needs to use en_US local
-export LANG=en_US.UTF-8
+export LANG=c
 
 CHECK_CMD_PKGS=(
     gcc
@@ -13,9 +13,8 @@ CHECK_CMD_PKGS=(
     lsb-release
     make
     net-tools
-    python-devel
-    python
-    venv
+    python3-devel
+    python3
     wget
 )
 
@@ -34,7 +33,6 @@ case ${ID,,} in
         [net-tools]=net-tools
         [python]=python
         [python-devel]=python-devel
-        [venv]=python-virtualenv
         [wget]=wget
     )
     EXTRA_PKG_DEPS=( python-xml )
@@ -60,12 +58,12 @@ case ${ID,,} in
         [lsb-release]=lsb-release
         [make]=make
         [net-tools]=net-tools
-        [python]=python-minimal
-        [python-devel]=libpython-dev
-        [venv]=python-virtualenv
+        [python3]=python3-minimal
+        [python3-devel]=libpython3-dev
         [wget]=wget
+        [venv]=python3-venv
     )
-    EXTRA_PKG_DEPS=()
+    EXTRA_PKG_DEPS=( python3-apt )
     sudo apt-get update
     ;;
 
@@ -74,36 +72,20 @@ case ${ID,,} in
     PKG_MANAGER=$(which dnf || which yum)
     INSTALLER_CMD="sudo -H -E ${PKG_MANAGER} -y install"
     CHECK_CMD="rpm -q"
-    if [ $(basename $PKG_MANAGER) = yum ]; then
-        PKG_MAP=(
-            [gcc]=gcc
-            [libffi]=libffi-devel
-            [libopenssl]=openssl-devel
-            [lsb-release]=redhat-lsb
-            [make]=make
-            [net-tools]=net-tools
-            [python]=python
-            [python-devel]=python-devel
-            [venv]=python-virtualenv
-            [wget]=wget
-        )
-    else
-        PKG_MAP=(
-            [gcc]=gcc
-            [libffi]=libffi-devel
-            [libopenssl]=openssl-devel
-            [lsb-release]=redhat-lsb
-            [make]=make
-            [net-tools]=net-tools
-            [python]=python3
-            [python-devel]=python3-devel
-            [venv]=python3-virtualenv
-            [wget]=wget
-        )
-    fi
+    PKG_MAP=(
+        [gcc]=gcc
+        [libffi]=libffi-devel
+        [libopenssl]=openssl-devel
+        [lsb-release]=redhat-lsb
+        [make]=make
+        [net-tools]=net-tools
+        [python3]=python3
+        [python3-devel]=python3-devel
+        [wget]=wget
+    )
     EXTRA_PKG_DEPS=()
     sudo -E ${PKG_MANAGER} updateinfo
-    if [ $(basename $PKG_MANAGER) = dnf ]; then
+    if $(grep -q Fedora /etc/redhat-release); then
         EXTRA_PKG_DEPS="python3-dnf redhat-rpm-config"
     fi
     ;;
@@ -115,18 +97,13 @@ esac
 # since it may already be present (but disabled) on the host
 if env | grep -q ^ZUUL; then
     if [ "${OS_FAMILY}" == "RedHat" ]; then
-        if [ $(basename $PKG_MANAGER) = yum ]; then
-            ${INSTALLER_CMD} yum-utils
-            sudo yum-config-manager --enable epel || true
-        else
-            ${INSTALLER_CMD} dnf-utils
-            sudo dnf config-manager --set-enabled epel || true
-        fi
+        ${INSTALLER_CMD} dnf-utils
+        sudo dnf config-manager --set-enabled epel || true
     fi
 fi
 
-if ! $(python --version &>/dev/null) && ! $(python3 --version &>/dev/null); then
-    ${INSTALLER_CMD} ${PKG_MAP[python]}
+if ! $(python3 --version &>/dev/null); then
+    ${INSTALLER_CMD} ${PKG_MAP[python3]}
 fi
 if ! $(gcc -v &>/dev/null); then
     ${INSTALLER_CMD} ${PKG_MAP[gcc]}
@@ -134,7 +111,7 @@ fi
 if ! $(wget --version &>/dev/null); then
     ${INSTALLER_CMD} ${PKG_MAP[wget]}
 fi
-if [ -n "${VENV-}" ]; then
+if [ -n "${VENV-}" -a "${OS_FAMILY}" == "Debian" ]; then
         ${INSTALLER_CMD} ${PKG_MAP[venv]}
 fi
 
@@ -156,11 +133,7 @@ if [ -n "${VENV-}" ]; then
     echo "NOTICE: Using virtualenv for this installation."
     if [ ! -f ${VENV}/bin/activate ]; then
         # only create venv if one doesn't exist
-        if which python 2>/dev/null; then
-            sudo -H -E python -m virtualenv --no-site-packages ${VENV}
-        else
-            sudo -H -E python3 -m venv --system-site-packages ${VENV}
-        fi
+        sudo -H -E python3 -m venv --system-site-packages ${VENV}
     fi
     # Note(cinerama): activate is not compatible with "set -u";
     # disable it just for this line.
@@ -174,7 +147,7 @@ fi
 
 # If we're using a venv, we need to work around sudo not
 # keeping the path even with -E.
-PYTHON=$(which python || which python3)
+PYTHON=$(which python3)
 
 # To install python packages, we need pip.
 #
@@ -182,42 +155,33 @@ PYTHON=$(which python || which python3)
 # older versions of pip are incompatible with
 # requests, one of our indirect dependencies (bug 1459947).
 #
-# Note(cinerama): We use pip to install an updated pip plus our
-# other python requirements. pip breakages can seriously impact us,
-# so we've chosen to install/upgrade pip here rather than in
-# requirements (which are synced automatically from the global ones)
-# so we can quickly and easily adjust version parameters.
-# See bug 1536627.
-#
-# Note(cinerama): If pip is linked to pip3, the rest of the install
-# won't work. Remove the alternatives. This is due to ansible's
-# python 2.x requirement.
-if [[ $(readlink -f /etc/alternatives/pip) =~ "pip3" ]]; then
-    sudo -H update-alternatives --remove pip $(readlink -f /etc/alternatives/pip)
+ls $PYTHON
+
+# workaround for PEP517 issue
+PYTHON_VER=$($PYTHON -V)
+if [[ $PYTHON_VER == "Python 3.6.8" ]]; then
+    sudo -H -E $PYTHON -m pip install pip==19.0
+    export PIP_OPTS=""
+else
+    sudo -H -E $PYTHON -m pip install -U pip --ignore-installed
+    export PIP_OPTS="--upgrade-strategy only-if-needed"
 fi
 
-if ! "${PYTHON}" -m pip > /dev/null; then
+if [ "$?" != "0" ]; then
     wget -O /tmp/get-pip.py https://bootstrap.pypa.io/3.4/get-pip.py
     sudo -H -E ${PYTHON} /tmp/get-pip.py
 fi
 
-PIP="${PYTHON} -m pip"
-
 if [ -n "${VENV-}" ]; then
   ls -la ${VENV}/bin
 fi
 
-# NOTE(dtantsur): 9.0.0 introduces --upgrade-strategy.
-sudo -H -E ${PIP} install --upgrade "pip>9.0"
-
-# upgrade setuptools, as latest version is needed to install some projects
-sudo -H -E ${PIP} install --upgrade --force setuptools
+PIP=$(echo $PYTHON | sed 's/python/pip/')
 
 if [ "$OS_FAMILY" == "RedHat" ]; then
     sudo -H -E ${PIP} freeze
     sudo -H -E ${PIP} install --ignore-installed pyparsing ipaddress
 fi
-
 sudo -H -E ${PIP} install -r "$(dirname $0)/../requirements.txt"
 
 # Install the rest of required packages using bindep
